using Azure.Cosmos;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.IdentityModel.Tokens.Jwt;
using System.Linq;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;
using TEAMModelOS.Models.Dto;
using TEAMModelOS.SDK.Models;
using TEAMModelOS.SDK;
using TEAMModelOS.SDK.Context.Constant.Common;
using TEAMModelOS.SDK.DI;
using TEAMModelOS.SDK.DI.AzureCosmos.Inner;
using TEAMModelOS.SDK.Extension;
using TEAMModelOS.SDK.Helper.Common.CollectionHelper;
using TEAMModelOS.SDK.Helper.Common.StringHelper;
using TEAMModelOS.Models;
using Microsoft.Extensions.Options;
using TEAMModelOS.SDK.Models.Cosmos;
using Microsoft.AspNetCore.Authorization;
using TEAMModelOS.Filter;
using StackExchange.Redis;
using TEAMModelOS.SDK.Models.Cosmos.Common.Inner;
using TEAMModelOS.Services.Common;
using System.IO;
using System.Dynamic;
using Azure.Storage.Blobs.Models;
using Azure.Storage.Sas;
using Lib.AspNetCore.ServerSentEvents;

namespace TEAMModelOS.Controllers.Core
{
    [ProducesResponseType(StatusCodes.Status200OK)]
    [ProducesResponseType(StatusCodes.Status400BadRequest)]
    //[Authorize(Roles = "HiTool")]
    [Route("hiscan")]
    [ApiController]
    public class HiScanController : ControllerBase
    {
        private readonly AzureRedisFactory _azureRedis;
        private readonly AzureCosmosFactory _azureCosmos;
        private readonly SnowflakeId _snowflakeId;
        private readonly AzureServiceBusFactory _serviceBus;
        private readonly DingDing _dingDing;
        private readonly Option _option;
        private readonly AzureStorageFactory _azureStorage;
        private readonly ServerSentEventsService _sse;
        public HiScanController(AzureCosmosFactory azureCosmos, AzureServiceBusFactory serviceBus, SnowflakeId snowflakeId, DingDing dingDing, IOptionsSnapshot<Option> option,
           AzureRedisFactory azureRedis, AzureStorageFactory azureStorage, ServerSentEventsService sse)
        {
            _azureCosmos = azureCosmos;
            _serviceBus = serviceBus;
            _snowflakeId = snowflakeId;
            _dingDing = dingDing;
            _option = option?.Value;
            _azureRedis = azureRedis;
            _azureStorage = azureStorage;
            _sse = sse;
        }

        ///<summary>
        ///查询教师的阅卷任务列表
        /// </summary>
        /// <data>
        ///    ! "code":"tmdid"
        /// </data>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        [HttpPost("verify-qrcode")]
       // [AuthToken(Roles = "teacher,admin")]
        public async Task<IActionResult> VerifyQrcode(JsonElement request)
        {
            try {
                if (!request.TryGetProperty("sid", out JsonElement sid)) return BadRequest();
                if (!request.TryGetProperty("id_token", out JsonElement id_token)) return BadRequest();
                
                IServerSentEventsClient sseClient;
                if (Guid.TryParse($"{sid}", out Guid guid) && (sseClient = _sse.GetClient(guid)) != null) {
                    //var clientName = sseClient.GetProperty<string>("NAME");
                    //var clientDID= sseClient.GetProperty<string>("DID");
                   // var isHiTeach = clientName.Contains("HiScan", StringComparison.OrdinalIgnoreCase);
                    var jwt = new JwtSecurityToken(id_token.GetString());
                    //TODO 此驗證IdToken先簡單檢查，後面需向Core ID新API，驗證Token
                    if (!jwt.Payload.Iss.Equals("account.teammodel", StringComparison.OrdinalIgnoreCase)) return BadRequest();
                    var id = jwt.Payload.Sub;
                    jwt.Payload.TryGetValue("name", out object name);
                    jwt.Payload.TryGetValue("picture", out object picture);

                    List<ScanSchool> schools = new List<ScanSchool>();
                   
                    //TODO 取得Teacher 個人相關數據(課程清單、虛擬教室清單、歷史紀錄清單等)，學校數據另外API處理，多校切換時不同
                    var client = _azureCosmos.GetCosmosClient();
                    var response = await client.GetContainer("TEAMModelOS", "Teacher").ReadItemStreamAsync(id, new PartitionKey("Base"));
                    int size = 0;
                    //老師個人資料(含初始化)
                    if (response.Status == 200)
                    {
                        var json = await JsonDocument.ParseAsync(response.ContentStream);
                        if (json.RootElement.TryGetProperty("schools", out JsonElement value))
                        {
                            if (json.RootElement.TryGetProperty("size", out JsonElement _size) && _size.ValueKind.Equals(JsonValueKind.Number))
                            {
                                size = _size.GetInt32();
                            }
                            foreach (var obj in value.EnumerateArray())
                            {
                                string statusNow = obj.GetProperty("status").ToString();
                                //正式加入才会有
                                if (statusNow == "join")
                                {
                                    //dynamic schoolExtobj = new ExpandoObject();
                                    var schoolJson = await client.GetContainer("TEAMModelOS", "School").ReadItemStreamAsync($"{obj.GetProperty("schoolId")}", new PartitionKey("Base"));
                                    var school = await JsonDocument.ParseAsync(schoolJson.ContentStream);
                                    var schoolId = obj.GetProperty("schoolId");
                                    var schoolName = obj.GetProperty("name");
                                    var schoolStatus = obj.GetProperty("status");
                                    var schoolPicture = school.RootElement.GetProperty("picture");
                                    //检查学校购买的模组是否包含阅卷模组
                                    int count = 0;
                                    string sql = $" SELECT value(count(serial))  FROM c join serial in c.serial where c.id ='{schoolId}'  and  c.pk='Product' and serial.prodCode='3222C6D2' ";
                                    await  foreach (var item in client.GetContainer("TEAMModelOS", "Teacher").GetItemQueryIterator<int>(sql, requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey("Product") })) {
                                        count = item;
                                    }
                                    if (count > 0) {
                                        //生成token
                                        var  token = JwtAuthExtension.CreateAuthToken(_option.HostName, id, name?.ToString(), picture?.ToString(), _option.JwtSecretKey, roles: new[] { "teacher" }, schoolID: $"{schoolId}");
                                        //获取线下阅卷评测

                                        schools.Add(new ScanSchool { name = $"schoolName", picture = $"{schoolPicture}", code = $"{schoolId}", status = $"{schoolStatus}" ,token=token});
                                    }
                                }
                            }
                        }

                        //換取AuthToken，提供給前端
                        var data = new { name, picture, id, schools };
                        await sseClient.SendEventAsync(data.ToJsonString());
                        return Ok(data);
                    }
                    else
                    {
                        return Ok(new { status = 404 });
                    }
                }
            } catch (Exception ex ) {
                await _dingDing.SendBotMsg($"IES5，{_option.Location}，hiscan/verify-qrcode()\n{ex.Message}{ex.StackTrace}", GroupNames.醍摩豆服務運維群組);
                return BadRequest();
            }
           return Ok();
        }
        private void GetExam(string schoolId ,string  tmdid)
        {


        }
    }

    public class ScanSchool { 
        public string name { get; set; }
        public string picture { get; set; }
        public string code { get; set; }
        public string status { get; set; }
        public string token { get; set; }
    }
}
